@@ -23,22 +23,39 @@
 #ifndef ITERATOR_H
 #define ITERATOR_H
 
-// reference on what needs to be implemented for STL iterators: http://stackoverflow.com/questions/7758580/writing-your-own-stl-container/7759622#7759622
+// reference on what needs to be implemented for STL iterators:
+// http://stackoverflow.com/questions/7758580/writing-your-own-stl-container/7759622#7759622
 
 //#include <iostream>
-//#include <iterator>
+// #include <iterator>
 
 /**
- * Iterator class for Eigen::Matrix. This file is included from EigenMatrixBaseAddons.h and the Iterator class is therefore not a member of the namespace mixt::.
- * It is accessible in MixtComp through the derived class mixt::Matrix.
+ * Iterator class for Eigen::Matrix. This file is included from
+ * EigenMatrixBaseAddons.h and the Iterator class is therefore not a member of
+ * the namespace mixt::. It is accessible in MixtComp through the derived class
+ * mixt::Matrix.
  */
 
-class Iterator: public std::iterator<std::random_access_iterator_tag, Scalar,
-		int, Scalar*, Scalar&> {
+class Iterator {
+
 public:
-	Iterator(int i, int j, Derived& mat) :
-			i_(i), j_(j), rows_(mat.rows()), p_mat_(&mat) {
-	}
+	// iterator traits
+	using iterator_category = std::random_access_iterator_tag;
+	using value_type = Scalar;
+	using difference_type = int;
+	using pointer = Scalar *;
+	using reference = Scalar &;
+
+	int i_;
+	int j_;
+	int rows_;
+	Derived *p_mat_;
+
+	Iterator(int i, int j, Derived &mat)
+		: i_(i),
+		  j_(j),
+		  rows_(mat.rows()),
+		  p_mat_(&mat) {}
 
 	Iterator operator+(int i) {
 		int posP, iP, jP;
@@ -48,11 +65,16 @@ public:
 		return Iterator(iP, jP, *p_mat_);
 	}
 
-	Iterator& operator+=(int i) {
+	Iterator operator+=(int i) {
 		posToIn(pos() + i, i_, j_);
 		return *this;
 	}
 
+	Iterator operator-=(int i) {
+		posToIn(pos() - i, i_, j_);
+		return *this;
+	}
+
 	Iterator operator-(int i) {
 		int posP, iP, jP;
 		posP = pos();
@@ -61,44 +83,40 @@ public:
 		return Iterator(iP, jP, *p_mat_);
 	}
 
-	int operator-(const Iterator& it) {
-		return pos() - it.pos();
-	}
+	int operator-(const Iterator &it) { return pos() - it.pos(); }
 
-	bool operator<(const Iterator& it) {
+	bool operator<(const Iterator &it) {
 		if (j_ < it.j_) {
 			return true;
 		} else if (j_ > it.j_) {
 			return false;
-		} else if (i_ < it.i_) // in this case, j_ == it.j_, hence comparison must be done on i_
-				{
+		} else if (i_ < it.i_) { // in this case, j_ == it.j_, hence comparison must be done on i_
 			return true;
 		}
 
 		return false;
 	}
 
-	bool operator==(const Iterator& it) {
+	bool operator==(const Iterator &it) {
 		if (i_ == it.i_ && j_ == it.j_)
 			return true;
 		else
 			return false;
 	}
 
-	bool operator>(const Iterator& it) {
+	bool operator>(const Iterator &it) {
 		if (j_ > it.j_) {
 			return true;
 		} else if (j_ > it.j_) {
 			return false;
-		} else if (i_ > it.i_) // in this case, j_ == it.j_, hence comparison must be done on i_
-				{
+		} else if (i_ > it.i_) { // in this case, j_ == it.j_, hence comparison must be done on i_
 			return true;
 		}
 
 		return false;
 	}
 
-	bool operator>=(const Iterator& it) {
+	bool operator>=(const Iterator &it) {
 		if (!(*this < it)) {
 			return true;
 		}
@@ -106,24 +124,20 @@ public:
 		return false;
 	}
 
-	bool operator!=(const Iterator& it) {
+	bool operator!=(const Iterator &it) {
 		if (i_ != it.i_ || j_ != it.j_)
 			return true;
 		else
 			return false;
 	}
 
-	Scalar& operator*() const {
-		return (*p_mat_)(i_, j_);
-	}
+	Scalar &operator*() const { return (*p_mat_)(i_, j_); }
 
-	Scalar* operator->() const {
-		return &(*p_mat_)(i_, j_);
-	}
+	Scalar *operator->() const { return &(*p_mat_)(i_, j_); }
 
-	Iterator& operator++() {
+	Iterator operator++() {
 		if (i_ < rows_ - 1) // row increment
-				{
+		{
 			++i_;
 		} else // column increment
 		{
@@ -133,7 +147,20 @@ public:
 		return *this;
 	}
 
-	Iterator& operator--() {
+	Iterator operator++(int) {
+		Iterator temp(*this);
+		if (i_ < rows_ - 1) // row increment
+		{
+			++i_;
+		} else // column increment
+		{
+			i_ = 0;
+			++j_;
+		}
+		return temp;
+	}
+
+	Iterator operator--() {
 		if (i_ > 0) {
 			--i_;
 		} else {
@@ -143,12 +170,7 @@ public:
 		return *this;
 	}
 
-	int i_;
-	int j_;
-	int rows_;
-	Derived* p_mat_;
-
-	void posToIn(int pos, int& i, int& j) const {
+	void posToIn(int pos, int &i, int &j) const {
 		std::div_t divresult;
 		divresult = std::div(pos, rows_);
 
@@ -156,9 +178,7 @@ public:
 		j = divresult.quot;
 	}
 
-	int pos() const {
-		return j_ * rows_ + i_;
-	}
+	int pos() const { return j_ * rows_ + i_; }
 };
 
 #endif // ITERATOR_H
